@@ -220,8 +220,9 @@ class Login2Request(WebQQRequest):
 
     def callback(self, resp, data):
         self.hub.require_check_time = None
-        if not resp.body:
-            logger.error(u"没有获取到数据, 登录失败")
+        if not resp.body or not isinstance(data, dict):
+            logger.error(u"没有获取到数据或数据格式错误, 登录失败:{0}"
+                         .format(resp.body))
             self.hub.load_next_request(FirstRequest())
             return
 
@@ -247,8 +248,9 @@ class FriendListRequest(WebQQRequest):
     url = "http://s.web2.qq.com/api/get_user_friends2"
     method = WebQQRequest.METHOD_POST
 
-    def init(self, first=True):
+    def init(self, first=True, manual=False):
         self.is_first = first
+        self.manual = manual
         self.params = [("r", json.dumps({"h": "hello",
                                          "hash": self.hub._hash(),
                                          "vfwebqq": self.hub.vfwebqq}))]
@@ -262,14 +264,21 @@ class FriendListRequest(WebQQRequest):
             logger.error("加载好友信息失败, 重新开始登录")
             return self.hub.load_next_request(FirstRequest())
 
-        info = data.get("result", {})
-        friends = self.hub.set_friends(info)
-        logger.info("加载好友信息 {0!r}".format(friends))
-        logger.info(data)
-        self.hub.load_next_request(GroupListRequest())
-        self.hub.load_next_request(FriendStatusRequest())
-        self.hub.load_next_request(DiscuListRequest())
-        self.hub.load_next_request(FriendListRequest(delay=3600, first=False))
+        if isinstance(data, dict) and data.get("retcode") == 0:
+            info = data.get("result", {})
+            self.hub.set_friends(info)
+            friends = self.hub.get_friends()
+            logger.info(u"加载好友信息 {0!r}".format(friends))
+            logger.debug(data)
+            self.hub.load_next_request(FriendStatusRequest())
+        else:
+            logger.warn(u"加载好友列表失败: {0!r}".format(data))
+
+        if not self.manual:
+            self.hub.load_next_request(GroupListRequest())
+            self.hub.load_next_request(DiscuListRequest())
+            self.hub.load_next_request(FriendListRequest(delay=3600,
+                                                         first=False))
 
 
 FriendInfoRequest = FriendListRequest
@@ -296,6 +305,8 @@ class FriendStatusRequest(WebQQRequest):
         if isinstance(data, dict) and data.get("retcode") == 0:
             for item in data.get('result', []):
                 self.hub.get_friends().set_status(**item)
+        else:
+            logger.warn(u"加载好友状态信息失败: {0!r}".format(data))
 
 
 class GroupListRequest(WebQQRequest):
@@ -341,12 +352,15 @@ class GroupMembersRequest(WebQQRequest):
         self.headers.update(Referer=const.S_REFERER)
 
     def callback(self, resp, data):
-        logger.debug(u"获取群成员信息 {0!r}".format(data))
-        members = data.get("result", {})
-        groups = self.hub.get_groups()
-        group = groups.find_group(self._gcode)
-        group.set_group_detail(members)
-        logger.debug(u"群详细信息: {0!r}".format(group))
+        if isinstance(data, dict) and data.get("retcode") == 0:
+            logger.debug(u"获取群成员信息 {0!r}".format(data))
+            members = data.get("result", {})
+            groups = self.hub.get_groups()
+            group = groups.find_group(self._gcode)
+            group.set_group_detail(members)
+            logger.debug(u"群详细信息: {0}".format(group))
+        else:
+            logger.warn(u"获取群成员信息失败 {0!r}".format(data))
 
         if self._poll:
             self.hub.start_poll()
@@ -388,6 +402,7 @@ class PollMessageRequest(WebQQRequest):
         self.ready = not self.hub.stop_poll
 
     def callback(self, resp, data):
+        polled = False
         try:
             if not data:
                 return
@@ -399,16 +414,20 @@ class PollMessageRequest(WebQQRequest):
 
             logger.info(u"获取消息: {0!r}".format(data))
             self.hub.load_next_request(PollMessageRequest())
+            polled = True
             self.hub.dispatch(data)
         except Exception as e:
             logger.error(u"消息获取异常: {0}".format(e), exc_info=True)
+        finally:
+            if not polled:
+                self.hub.load_next_request(PollMessageRequest())
 
 
 class SessGroupSigRequest(WebQQRequest):
 
     """ 获取临时消息群签名请求
 
-    :param qid: 临时签名对应的qid
+    :param qid: 临时签名对应的qid(对应群的gid)
     :param to_uin: 临时消息接收人uin
     :param sess_reqeust: 发起临时消息的请求
     """
@@ -544,6 +563,8 @@ class QQNumberRequest(WebQQRequest):
             uin = r.get("uin")
             account = r.get("account")
             self.hub.get_friends().set_account(uin, account)
+        else:
+            logger.warn(u"获取好友QQ号码失败: {0!r}".format(data))
 
 
 class DiscuInfoRequest(WebQQRequest):
@@ -565,7 +586,7 @@ class DiscuInfoRequest(WebQQRequest):
     def callback(self, resp, data):
         if data.get("retcode") == 0:
             discu = self.hub.get_discu()
-            logger.info("获取讨论组 {0} 的详细信息: {1}"
+            logger.info(u"获取讨论组 {0} 的详细信息: {1}"
                         .format(discu.get_name(self._did), data))
             discu.set_detail(self._did, data.get("result", {}))
 
@@ -672,8 +693,8 @@ class AcceptVerifyRequest(WebQQRequest):
     def callback(self, resp, data):
         if data.get("retcode") == 0:
             logger.info(u"添加 {0} 成功".format(self.qq_num))
-            if self.markname:
-                self.hub.mark_to_uin[self.markname] = self.uin
+            # if self.markname:
+            #     self.hub.mark_to_uin[self.markname] = self.uin
         else:
             logger.info(u"添加 {0} 失败".format(self.qq_num))
 
@@ -710,6 +731,22 @@ class FileRequest(WebQQRequest):
             self._cb(self.fname, response.body)
 
 
+class LogoutRequset(WebQQRequest):
+    """ 登出请求
+    """
+    url = "https://d.web2.qq.com/channel/logout2"
+
+    def init(self):
+        self.params = {"clientid": self.hub.clientid, "ids": "",
+                       "psessionid": self.hub.psessionid,
+                       "t": int(time.time() * 1000)}
+        self.headers.update(Referer=const.D_REFERER)
+
+    def callback(self, resp, data):
+        if data.get("retcode") == 0:
+            logger.info(u"登出成功")
+
+
 FirstRequest = LoginSigRequest
 
 
